async function main() {
    const path = require("path");
    const fs = require("fs");
    const winston = require("winston");
    const argv = require("minimist")(process.argv.slice(2));

    // Activate colorization for output
    winston.remove(winston.transports.Console);
    winston.add(winston.transports.Console, {
        colorize: true,
        level: "info",
        json: false,
        stringify: true,
    });

    // Setup variables to find the Makefile location
    const command = argv._[0] || "all";
    const args = argv._.slice(1);
    let fileName = "Makefile.js";
    const cwd = process.cwd();
    const packageJsonPath = path.resolve(cwd, "package.json");

    if (fs.existsSync(packageJsonPath)) {
        try {
            const packageJson = require(packageJsonPath);

            if (typeof packageJson.jmake === "string") {
                fileName = packageJson.jmake;
            }
        } catch (e) {
            winston.warn(`Unable to require '${packageJsonPath}'.`);
            winston.warn(`Defaulting fileName to ${fileName}.`);
            winston.warn(`Error: ${e.message}.`);
        }
    }

    if ("f" in argv || "file" in argv) {
        fileName = argv.f || argv.file;
    }

    const makefilePath = path.resolve(cwd, fileName);

    // Retrieve Makefile
    let makefile;
    try {
        makefile = require(makefilePath);
    } catch (e) {
        winston.error(`Unable to require '${makefilePath}'.`);
        winston.error(`Error: ${e.message}.`);
        winston.error("Can't do anything. Exiting...");
        process.exit(1);
    }

    // Do the processing
    if (command in makefile) {
        const handler = makefile[command];

        if (typeof handler !== "function") {
            winston.error("The command must be a function.");
            process.exit(4);
        }

        const errorHandler = e => {
            winston.error("==============================================================");
            winston.error("|     Something went wrong while executing the command.      |");
            winston.error("==============================================================");
            winston.error(e.message);
            winston.error("==============================================================");
            winston.error(e);
            winston.error("==============================================================");
            process.exit(3);
        };

        try {
            const result = handler(...args);
            Promise.resolve(result).catch(errorHandler);
        } catch (e) {
            errorHandler(e);
        }
    } else if (command === "-h" || command === "--help") {
        winston.info("Available commands:");
        Object.keys(makefile)
            .filter(cmd => typeof makefile[cmd] === "function")
            .map(cmd => `\t- ${cmd}`)
            .forEach(cmd => {
                winston.info(cmd);
            });
        process.exit();
    } else {
        winston.error(`The command '${command}' does not exist in your Makefile.`);
        process.exit(2);
    }
}

module.exports = main;
